//*********************************************************************************************************
// File: moonpos.cpp
// Description:
// This application generates stepper motor commands for the VEX Moonbot application.  
// It reads the sunpos.dat file for Azimuth and Elevation angles and converts them to
// stepper motor commands which are written to the moonpos.out file. Just insert this file 
// into your VEX PIC18 C SUNBOT application.

// Use GNU C++
// Compile from a DOS shell using the following command:
// g++ -o teststepper teststepper.cpp graphuti.cpp

//*********************************************************************************************************

#include <stdio.h>
#include <math.h>
#include "sensor.h"
#include "graphuti.h"
 

#define Total_Stepper_Readings  20  // Total number of stepper motor calibration readings
#define Total_Commands  69    // Total number of stepper motor commands for the current day
#define MAXINT 65535 

// Data generated from stepper motor calibration process
int Azimuth_Stepper_Readings[Total_Commands]   = {801,798,805,805,798,798,801,798,798,800,799,801,798,799,798,804,801,803,800,801 }; // SunBot III
//int Azimuth_Stepper_Readings[Total_Stepper_Readings]   = {1825,1797,1826,1831,1858,1831,1889,1828,1855,1828,1828,1859,1860,1826,1828,1857,1860,1827,1887,1825 }; // SunBot II
int Azimuth_Encoder_Readings[Total_Commands]   = {38,37,34,34,40,37,39,39,38,38,39,38,40,39,37,39,39,37,39,39  };// SunBot II

int Elevation_Stepper_Readings[Total_Commands] = {402,401,400,401,406,400,401,403,401,400,401,403,403,400,400,400,400,401,405,401};// SunBot III
//int Elevation_Stepper_Readings[Total_Stepper_Readings] = {1531,1533,1523,1522,1536,1543,1536,1526,1527,1541,1541,1533,1527,1533,1527,1525,1535,1540,1541,1538};// SunBot II
int Elevation_Encoder_Readings[Total_Commands] = {24,25,22,21,23,23,24,24,22,23,22,21,23,21,24,23,22,24,25,23};// SunBot II


int Azimuth_Stepper_Commands[Total_Commands];
int Elevation_Stepper_Commands[Total_Commands];

float Azimuth_Angles[Total_Commands];
float Elevation_Angles[Total_Commands];

// Declare function headers here
int main(void);
int Min_Vector(int Vector[], int Number_Of_Elements);
int Max_Vector(int Vector[], int Number_Of_Elements);
float Average_Vector(int Vector[], int Number_Of_Elements);
FILE *f1, *f2;
int Line = 0;
int j;

int main(void)
{
	int i;
    float Stepper_Counts;
    float Fraction_Illuminated;
    int Stepper_Commands;
	char Comment[80];
	char Time[5];
    char Data[80];


/*
	for (i=0; i<1000; i++)
	{
		printf(" This is a test %d times !!!\r\n", i);  
	}
*/
#ifdef AZIMUTH

	// Calibrate stepper motors using quadrature optical encoders and limit switches
    printf("Minimum Azimuth Stepper Reading = %d \r\n", Min_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings));
    printf("Minimum Azimuth Encoder Reading = %d \r\n", Min_Vector(Azimuth_Encoder_Readings, Total_Stepper_Readings));
    printf("Maximum Azimuth Stepper Reading = %d \r\n", Max_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings));
    printf("Maximum Azimuth Encoder Reading = %d \r\n", Max_Vector(Azimuth_Encoder_Readings, Total_Stepper_Readings));
    printf("Average Azimuth Stepper Reading = %f \r\n", Average_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings));
    printf("Average Azimuth Encoder Reading = %f \r\n", Average_Vector(Azimuth_Encoder_Readings, Total_Stepper_Readings));

	// Set global viewport coordinates here for gain and offset calibration of Azimuth axis
	set_view(0, 0,
             (int) Average_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings), (int) Average_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings));

	// Set the global window  coordinates here for gain and offset calibration of Azimuth axis
	//set_window(0.0, 0.0, 360.0, 360.0);
	set_window(0.0, 0.0, 180.0, 180.0);
 
	for (i=1; i<=360; i++)
	{
		// Map the window coordinates (Azimuth angles) to viewport coordinates (stepper motor commands)
		map_window_to_viewport((float)i, (float)i, &Stepper_Commands, &Stepper_Commands);
     	printf("Azimuth Angle = %d  Azimuth Stepper Commands = %d\r\n", i, Stepper_Commands);
	}

#endif
 


#ifdef ELEVATION


	// Calibrate stepper motors using quadrature optical encoders and limit switches
    printf("Minimum Elevation Stepper Reading = %d \r\n", Min_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));
    printf("Minimum Elevation Encoder Reading = %d \r\n", Min_Vector(Elevation_Encoder_Readings, Total_Stepper_Readings));
    printf("Maximum Elevation Stepper Reading = %d \r\n", Max_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));
    printf("Maximum Elevation Encoder Reading = %d \r\n", Max_Vector(Elevation_Encoder_Readings, Total_Stepper_Readings));
    printf("Average Elevation Stepper Reading = %f \r\n", Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));
    printf("Average Elevation Encoder Reading = %f \r\n", Average_Vector(Elevation_Encoder_Readings, Total_Stepper_Readings));

	// Set global viewport coordinates here for gain and offset calibration of elevation axis
	set_view(0, 0,
             (int) Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings), (int) Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));

	// Set the global window  coordinates here for gain and offset calibration of elevation axis
	set_window(0.0, 0.0, 90.0, 90.0); 
 
	for (i=1; i<=180; i++)
	{
		// Map the window coordinates (elevation angles) to viewport coordinates (stepper motor commands)
		map_window_to_viewport((float)i, (float)i, &Stepper_Commands, &Stepper_Commands);
     	printf("Elevation Angle = %d  Elevation Stepper Commands = %d\r\n", i, Stepper_Commands);
	}


#endif

#ifndef VEX_CODE

	// Read the data from a text containing Sun Position data 
    f1 = fopen("moonpos.dat", "r");

	// Open output file
	f2 = fopen("moonpos.out", "w");

	fprintf(f2,"// Insert this code containing Azimuth and Elevation (Altitude) stepper commands\r\n");
    fprintf(f2,"// in the declaration section of your VEX PIC18 C application and re-compile the application...\r\n");
	fprintf(f2,"#define Total_Commands   %d \r\n", Total_Commands);

	// Read the header information and copy it to the output file as a C comment

    fprintf(f2,"/*\r\n");
  	for (i=0; i<45; i++)
	{
        fscanf(f1, "%80s", Comment);
        fprintf(f2, "%80s \r\n", Comment);
	}
    fprintf(f2,"*/\r\n");
  
	// Read and parse the data information and copy it to the output file as C code

   	for (j=0; j< (int)Total_Commands; j++)
	{

        //fscanf(f1, "%5s %5,1f, %5,1f", Time, &Azimuth_Angles[i], &Elevation_Angles[i]);
        //fprintf(f2, "%5s %5,1f, %5,1f \r\n", Time, &Azimuth_Angles[i], &Elevation_Angles[i]);
        fscanf(f1, "%5s", Time);
        fscanf(f1, "%f", &Azimuth_Angles[j]);
        fscanf(f1, "%f", &Elevation_Angles[j]);
        fscanf(f1, "%f", &Fraction_Illuminated);
     
		printf(     "main - trace 1: %d, %5s, %f, %f %f\r\n", j, Time, Azimuth_Angles[j], Elevation_Angles[j], Fraction_Illuminated);
		fprintf(f2, "main - trace 1: %d, %5s, %f, %f %f\r\n", j, Time, Azimuth_Angles[j], Elevation_Angles[j], Fraction_Illuminated);

        //fscanf(f1, "%80s", Data);
        //fprintf(f2, "%80s \r\n", Data);

        //if (Line >Total_Commands) break;
		// Increment line counter
		//Line++;
        
	}

	// Close the data file
    fclose(f1);
 
#ifndef NEWCODE

	// Set global viewport coordinates here for gain and offset calibration of Azimuth axis
	set_view(0, 0,
             (int) Average_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings), (int) Average_Vector(Azimuth_Stepper_Readings, Total_Stepper_Readings));

	// Set the global window  coordinates here for gain and offset calibration of Azimuth axis
	//set_window(0.0, 0.0, 360.0, 360.0);
	set_window(0.0, 0.0, 180.0, 180.0);

	for (i=0; i< (int)Total_Commands; i++)
	{
        if (Azimuth_Angles[i] > 0.0)
        {
			// Map the window coordinates (Azimuth angles) to viewport coordinates (stepper motor commands)
			map_window_to_viewport(Azimuth_Angles[i], Azimuth_Angles[i], &Azimuth_Stepper_Commands[i], &Azimuth_Stepper_Commands[i]);
     		printf("Azimuth Angle = %f  Azimuth Stepper Commands = %d\r\n", Azimuth_Angles[i], Azimuth_Stepper_Commands[i]);
  		}
		else
		{
            // Don't move the azimuth stepper motor for negative angles
			Azimuth_Stepper_Commands[i] = 0;
		}
	}

	// Set global viewport coordinates here for gain and offset calibration of elevation axis
	set_view(0, 0,
             (int) Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings), (int) Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));


	// Set the global window  coordinates here for gain and offset calibration of elevation axis
	set_window(0.0, 0.0, 90.0, 90.0); 

	for (i=0; i< (int) Total_Commands; i++)
	{

        if (Elevation_Angles[i] > 0.0)
        {
			// Map the window coordinates (elevation angles) to viewport coordinates (stepper motor commands)
			map_window_to_viewport(Elevation_Angles[i], Elevation_Angles[i], &Elevation_Stepper_Commands[i], &Elevation_Stepper_Commands[i]);
     		printf("Elevation Angle = %f  Elevation Stepper Commands = %d\r\n", Elevation_Angles[i], Elevation_Stepper_Commands[i]);
		}
		else
		{
            // Don't move the elevation stepper motor for negative angles
			Elevation_Stepper_Commands[i] = 0;
		}
	}
 
	fprintf(f2,"int Azimuth_Stepper_Commands[Total_Commands] = {\r\n");

	// Generate code for VEX Microcontroller to initialize Astronomical Data
	for (i=0; i<Total_Commands-1; i++)
	{
		fprintf(f2, "%d, ", Azimuth_Stepper_Commands[i]);
	}
	fprintf(f2, "%d};\r\n", Azimuth_Stepper_Commands[Total_Commands-1]);

	fprintf(f2,"int Elevation_Stepper_Commands[Total_Commands] = {\r\n");

	// Generate code for VEX Microcontroller to initialize Astronomical Data
	for (i=0; i<Total_Commands-1; i++)
	{
		fprintf(f2, "%d, ", Elevation_Stepper_Commands[i]);
	}
	fprintf(f2, "%d};\r\n", Elevation_Stepper_Commands[Total_Commands-1]);

#endif

	// Close the VEX output file
    fclose(f2);

#endif

#ifdef NEW_CODE
	// Set global viewport coordinates here for gain and offset calibration of elevation axis
	set_view(0, 0, 180, 180); 

	// set global window coordinates here for elevation axis
	//set_window((float)Min_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings), (float)Min_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings),
    //           (float)Max_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings), (float)Max_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));
	//set_window((float)0, (float)0,
    //           (float)Max_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings), (float)Max_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));

	// Set the global window  coordinates here for gain and offset calibration of elevation axis
	set_window((float)0, (float)0,
                Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings), Average_Vector(Elevation_Stepper_Readings, Total_Stepper_Readings));

/*
	for (i=0; i<180; i++)
	{
		// Generate the Elevation stepper motor commands
		//map_viewport_to_window(i, i, &Stepper_Counts, &Stepper_Counts);
		map_window_to_viewport((float)i, (float)i, &Stepper_Counts, &Stepper_Counts);
     	printf("Elevation Angle = %d  Elevation Stepper Counts = %f\r\n", i, Stepper_Counts);
	}
*/


	for (i=1; i<=180; i++)
	{
		// Map the viewport coordinates (elevation angles) to window coordinates (stepper motor commands)
		map_viewport_to_window(i, i, &Stepper_Counts, &Stepper_Counts);
     	printf("Elevation Angle = %d  Elevation Stepper Counts = %f\r\n", i, Stepper_Counts);
	}
#endif

	return 0;
	
}

//******************************************************************************
//* Min_Vector  - Return the minimum positive value of the Vector
//******************************************************************************
int  Min_Vector(int Vector[], int Number_Of_Elements)
{
	int i;
	int Minimum_Vector_Value = MAXINT;

	Minimum_Vector_Value = MAXINT;
	for (i=0; i<Number_Of_Elements; i++)
	{
		if (Vector[i] < Minimum_Vector_Value)
			Minimum_Vector_Value = Vector[i];

	}
	// Return the minimum value of the Vector
	return Minimum_Vector_Value;

}

//******************************************************************************
//* Max_Vector  - Return the maximum positive value of the Vector
//******************************************************************************
int Max_Vector(int Vector[], int Number_Of_Elements)
{
	int i;
	int Maximum_Vector_Value = MAXINT;

	Maximum_Vector_Value = -1;
	for (i=0; i<Number_Of_Elements; i++)
	{
		if (Vector[i] > Maximum_Vector_Value)
			Maximum_Vector_Value = Vector[i];

	}
	// Return the maximum value of the Vector
	return Maximum_Vector_Value;

}
	
//******************************************************************************
//* Average_Vector  - Return the Mean value of the Vector as a floating point value
//******************************************************************************
float Average_Vector(int Vector[], int Number_Of_Elements)
{
	int i;
	float Average_Vector_Value = 0.0;
    float Sum = 0.0;

    Sum = 0.0;
 
	for (i=0; i<Number_Of_Elements; i++)
	{
		Sum += (float)Vector[i];
	}
	Average_Vector_Value = Sum / (float)Number_Of_Elements;

	// Return the maximum value of the Vector
	return Average_Vector_Value;

}
	